#include <propeller.h>
#include <stdlib.h>
#include <stdint.h>
#include <stdio.h>
#include "Domain\SchedulerRegistry.h"
#include "Lamps\LampShow.h"
#include "CogC_Drivers\IODriver\IODriverMailbox.h"

#define MAX_LOOP_COUNTER 1

extern "C" void __cxa_pure_virtual() {
  while (1);
}

extern _Driver _SimpleSerialDriver;
extern _Driver _FileDriver;

/* Driver list */
_Driver *_driverlist[] = {
    &_SimpleSerialDriver,
    &_FileDriver,
    NULL
};

// THIS IS THE PROBLEMATIC PART:
const uint32_t testIndex = 8000;
volatile uint8_t testArr[testIndex];

static struct {
    uint32_t ioDriverStack[IO_DRIVER_STACK_SIZE];
    volatile ioDriverMailbox_t ioDriverMailbox;
} ioDriverPar;

SchedulerRegistry schedulerRegistry;

// Scheduler cog stack declaration
static uint8_t schedulerCogStack[EXTRA_STACK_BYTES + 2000];       // allocate space for the stack - you need to set up a
                                                                  // stack for each Cog you plan on starting

/*
 * Function to start up a new cog running the IO driver
 * code (which we have placed in the IODriver.cog section)
 */
uint8_t startIODriver(volatile void *parptr) {
  extern unsigned int _load_start_IODriver_cog[];
  #if defined(__PROPELLER_XMM__) || defined(__PROPELLER_XMMC__)
    uint32_t hub_buffer[496];
    extern unsigned int _load_stop_IODriver_cog[];
    memcpy(hub_buffer, _load_start_IODriver_cog, (_load_stop_IODriver_cog - _load_start_IODriver_cog) * sizeof(uint32_t));
    return cognew(hub_buffer, parptr);
  #else
    return cognew(_load_start_IODriver_cog, parptr);
  #endif
}

void schedulerCog(void* par) {
  // Schedule tasks.
  while(1) {
    printf("In scheduler loop...\n");
    schedulerRegistry.schedule();
//    waitcnt((1) + CNT);
  }
}

int main (void) {
  LampShow lampShow((uint8_t*) ioDriverPar.ioDriverMailbox.lampState, (uint8_t*) ioDriverPar.ioDriverMailbox.outputPort);
  schedulerRegistry.addScheduler(lampShow, 31);

  waitcnt(CLKFREQ + CNT);

  // Start the IO driver cog.
  startIODriver(&ioDriverPar.ioDriverMailbox);
  cogstart(schedulerCog, NULL, schedulerCogStack, sizeof(schedulerCogStack));
  

  for (uint32_t y = 0; y < testIndex; y++) {
    testArr[y] = 1;
  }

  while (1) {
    waitcnt((CLKFREQ / 4) + CNT);
  }
}
